/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2003   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c   (for Mitsubishi M16C and TASKING compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef OS_FSYS                     /* CPU main clock frequncy      */
  #define OS_FSYS 16000000L
#endif

#if (defined _REGM306NB_SFR)  || (defined _REGM306NA_SFR)  \
 || (defined _REGM306N1_SFR)  || (defined _REGM306N0_SFR)  \
 || (defined _REGM30627P_SFR) || (defined _REGM30626P_SFR) \
 || (defined _REGM30625P_SFR) || (defined _REGM30624P_SFR) \
 || (defined _REGM30623P_SFR) || (defined _REGM30622P_SFR) \
 || (defined _REGM30620P_SFR)                              \
 || (defined _REGM30291_SFR)  || (defined _REGM30290_SFR)  \
 || (defined _REGM30281_SFR)  || (defined _REGM30200_SFR)  \
 || (defined _REGM30260A_SFR) || (defined _REGM30262_SFR)  \
 || (defined _REGM30263A_SFR)
  /* Initially Peripheral clock is main clock / 2 for this CPUs     */
  #define OS_PCLK (OS_FSYS / 2)
#else
  #define OS_PCLK OS_FSYS           /* Peripheral clock=main clock  */ 
#endif

#ifndef OS_PCLK_TIMER               /* Peripheral clock for timer   */ 
  #define OS_PCLK_TIMER OS_PCLK     /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef OS_PCLK_UART                /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_PCLK      /* May vary from CPU clock      */
#endif                              /* depending on CPU             */
  
/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/

#ifndef OS_UART
  #define OS_UART 0
#endif    
#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*       OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {
  PRCR   = 0x1;            /* Enable writing of sys. clock registers.*/
  CM16   = 0x0;	           /* System clock to full speed             */
  CM17   = 0x0;	           /* System clock to full speed             */
  CM06   = 0x0;	           /* System clock set to CM16/CM17 settings */
  PRCR   = 0x0;            /* Disable writing of sys clock registers.*/
  TA0    = OS_PCLK_TIMER/1000-1;  /* This value is for 1ms Tick.     */
  TA0MR  = 0;              /* Timer mode register                    */
                           /* bit 01 : 00 Timer mode                 */
                           /* bit 2  :  0 No puls output             */
                           /* bit 34 : 0X No gate function           */
                           /* bit 5  : 0                             */
                           /* bit 67 : 00 clock source : fx/2        */
  TA0IC  = 1;              /* Timer interrupt enable, priority 1     */
  TABSR |= 1;              /* Timer count start flag                 */
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {      /* Idle loop: No task is ready to exec */
  for (;;);
}

/*********************************************************************
*
*       Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer
    or high precision time measurment functions.
    It returns the system time in clock cycles.
*/

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TA0;
  OS_U32 time = OS_Time+1;
  if (TA0IC & (1 << 3)) {  /* if timer interrupt pending */
    t_cnt = TA0;           /* Adjust result              */
    time++;
  }
  return (OS_PCLK_TIMER/1000) * time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert Cycles into micro seconds. (For profiling only)

   If your clock frequency is not a multiple of 1 MHz,
   you may have to modify this routine in order to get proper
   diagonstics.
Please note:
   This routine is required for profiling only.
   It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*
**********************************************************************
When using an other hardware timer as embOS timer tick, always set
the correct interrupt vector here
*/

__interrupt (21) void OS_ISR_Tick (void) {
  OS_TickHandler();
}

/*********************************************************************
*
*       Communication for embOSView   (UART 0)
*
**********************************************************************
*/

#define BAUDDIVIDE ((OS_U8)((OS_PCLK_UART+OS_BAUDRATE*8L)/(OS_BAUDRATE*16L))-1)

#if (OS_UART == 0)

void OS_COM_Init(void) {
  U0MR   = 0x00;        /*  lock Sio, error reset              */
  U0C0   = 0x10;        /*  RTS/CTS disabled, clock divisor 1  */
  U0BRG  = BAUDDIVIDE;  /*  Calculated Baudrate                */
  U0C1   = 0;           /*  Lock Rx and Tx                     */
  PD6   &= ~(1<<2);     /*  RxD Port set as input              */
  PD6   &= ~(1<<3);     /*  TxD Port set as input              */
  U0MR   = 0x05         /*  8  Data                            */
          +(0<<4)       /*  0: 1 Stop-Bit (1= 2 Stop-Bits)     */
          +(0<<5)       /*  X: Parity select                   */
          +(0<<6)       /*  1: parity enable                   */
          +(0<<7);      /*  0: no sleep                        */
  U0C1   = 5;           /*  Enable reception and transmission  */
  S0RIC  = 1;           /*  Enable UART Rx interrupts          */
  S0TIC  = 1;           /*  Enable UART Tx interrupts          */
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/

void OS_COM_Send1(OS_U8 c) {
  U0TBL = c;
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART0
*/

__interrupt (17) void OS_ISR_tx(void) {
  OS_EnterNestableInterrupt();
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART0
*/

__interrupt (18) void OS_ISR_rx(void) {
  int Data;
  OS_EnterNestableInterrupt();  /* We will enable interrupts */
  Data = U0RB;
  if (Data & 0x6000) {          /* Check if errors occurred  */
    U0C1 &= 255-(1<<2);         /* disable Rx                */
    U0C1 |=  (1<<2);            /* enable Rx => error reset  */
  } else {
    OS_OnRx((OS_U8) Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView   (UART 1)
*
**********************************************************************
When using UART 1 for embOS, debugging via ROM-Monitor is impossible
*/

#elif (OS_UART == 1)

void OS_COM_Init(void) {
  U1MR   = 0x00;        /*  lock Sio, error reset              */
  U1C0   = 0x10;        /*  RTS/CTS disabled, clock divisor 1  */
  U1BRG  = BAUDDIVIDE;  /*  Calculated Baudrate                */
  U1C1   = 0;           /*  Lock Rx and Tx                     */
  PD6   &= ~(1<<6);     /*  RxD Port set as input              */
  PD6   &= ~(1<<7);     /*  TxD Port set as input              */
  U1MR   = 0x05         /*  8  Data                            */
          +(0<<4)       /*  0: 1 Stop-Bit (1= 2 Stop-Bits)     */
          +(0<<5)       /*  X: Parity select                   */
          +(0<<6)       /*  1: parity enable                   */
          +(0<<7);      /*  0: no sleep                        */
  U1C1   = 5;           /*  enable reception and transmition   */
  S1RIC  = 1;           /*  Enable UART Rx interrupts          */
  S1TIC  = 1;           /*  Enable UART Tx interrupts          */
}

/*********************************************************************
*
*       OS_COM_Send1() for UART1: Send one byte to embOSView
*
*       NEVER call this function from your application
*/

void OS_COM_Send1(OS_U8 c) {
  U1TBL = c;
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART1
*/

__interrupt (19) void OS_ISR_tx(void) {
  OS_EnterNestableInterrupt();
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART1
*/

__interrupt (20) void OS_ISR_rx(void) {
  int Data;
  OS_EnterNestableInterrupt();  /* We will enable interrupts */
  Data = U1RB;
  if (Data & 0x6000) {          /* Check if errors occurred  */
    U1C1 &= 255-(1<<2);         /* disable Rx                */
    U1C1 |=  (1<<2);            /* enable Rx => error reset  */
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/

#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }
#endif /* (OS_UART==0) || (OS_UART==1) */

/*****  EOF  ********************************************************/

